#! /usr/bin/guile \
--debug -e main -s
!#

;;;;
;;;; block-net-rename
;;;;
;;;; Copyright 2006 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-geda.
;;;;
;;;; octw-guile-geda is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-geda is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-geda.  If not, see <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: block-net-rename.scm,v 1.14 2008-09-08 18:54:39 dean Exp $
;;;; Dean Ferreyra

;;;; The purpose of block-net-rename is to take a template block
;;;; schematic and "instantiate" a specific schematic using the given
;;;; number.  It does this by replacing instances of "{n}" with the
;;;; given schematic number.  For example, if the net name is VALVE{n},
;;;; and the given schematic number is 2, you'll get VALVE2

(use-modules (srfi srfi-1))
(use-modules (ice-9 regex))
(use-modules (ice-9 getopt-long))
(use-modules (octw conditional) (octw anaphoric))
(use-modules (octw file-system) (octw gaf))

(define *cmd* "block-net-rename.scm")

(define (usage)
  (format #t "usage: ~A [OPTIONS]... <gschem-file>~%" *cmd*)
  (format #t "  -n, --number=NUMBER  schematic number~%")
  (format #t "  -t, --type=sch|sym   file type~%")
  (format #t "  -o, --output=FILE    output file~%")
  (format #t "  -h, --help           this usage message~%")
  (exit 0))

(define (display-error msg)
  (format #t "~A: ~A~%" *cmd* msg)
  (exit 1))

(define option-spec
  `((output (single-char #\o) (value #t))
    (type (single-char #\t) (value #t))
    (number (single-char #\n) (value #t))
    (help (single-char #\h))))

(define match-assoc
  `((".sch" . ("refdes"
               "block-rename"))
    (".sym" . ("pinlabel"
               "source"
               "block-rename"))))

(define (main args)
  (let ((options (getopt-long (command-line) option-spec)))
    (when (option-ref options 'help #f)
      (usage))
    (let ((files (option-ref options '() '())))
      (unless (= (length files) 1)
        (display-error "Only one file expected"))
      (let ((file-name (first files))
            (schematic-number (option-ref options 'number #f)))
        (unless (string->number schematic-number)
          (display-error "Schematic number expected"))
        (let* ((type (option-ref options 'type #f))
               (explicit-match-list
                (aand type
                      (assoc (string-append "." type) match-assoc)
                      (cdr it))))
          (when (and type (not explicit-match-list))
            (display-error (format #f "Unknown type given: ~S" type)))
          (let ((deduced-match-list 
                 (aand (assoc (extname file-name) match-assoc)
                       (cdr it))))
            (let ((match-list (or explicit-match-list deduced-match-list)))
              (unless match-list
                (display-error "Could not determine type"))
              (call-with-input-file file-name
                (lambda (in)
                  (aif (option-ref options 'output #f)
                       (call-with-output-file it
                         (lambda (out)
                           (run in schematic-number match-list out)))
                       (run in schematic-number match-list
                            (current-output-port))))))))))))

(define extname
  (let ((rx (make-regexp "(\\..*)")))
    (lambda (file)
      (or
       (aand (regexp-exec rx file)
             (match:substring it 1))
       ""))))

(define *property-rx* (make-regexp "^[[:space:]]*(.*)="))

(define (run in schematic-number match-list out)
  (line-by-line
   in
   (lambda (line line-number k)
     (display (line-subst schematic-number line match-list) out))))

(define (line-subst schematic-number line match-list)
  (let ((match (regexp-exec *property-rx* line)))
    (if match
        (let ((property (match:substring match 1)))
          (let loop ((match-list match-list))
            (acond ((null? match-list)
                    line)
                   ((string=? (car match-list) property)
                    (regexp-substitute/global #f
                                              "[{]n[}]"
                                              line
                                              'pre schematic-number 'post))
                   (else
                    (loop (cdr match-list))))))
        line)))
